/*
 * Beangle, Agile Java/Scala Development Scaffold and Toolkit
 *
 * Copyright (c) 2005-2012, Beangle Software.
 *
 * Beangle is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Beangle is distributed in the hope that it will be useful.
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Beangle.  If not, see <http://www.gnu.org/licenses/>.
 */
package org.beangle.commons.orm.hibernate;

import java.io.IOException;

import javax.servlet.FilterChain;
import javax.servlet.ServletException;
import javax.servlet.ServletRequest;
import javax.servlet.ServletResponse;

import org.beangle.commons.orm.hibernate.internal.SessionUtils;
import org.beangle.commons.web.filter.OncePerRequestFilter;
import org.beangle.commons.web.spring.ContextLoader;
import org.beangle.commons.web.spring.LazyInitializingHook;
import org.hibernate.SessionFactory;
import org.springframework.context.ApplicationContext;

public class OpenSessionInViewFilter extends OncePerRequestFilter implements LazyInitializingHook {

  private SessionFactory sessionFactory;

  @Override
  protected void initFilterBean() throws ServletException {
    super.initFilterBean();
    ApplicationContext context = ContextLoader.getContext(getServletContext());
    if (null != context) {
      setSessionFactory(context.getBean(SessionFactory.class));
    } else {
      ContextLoader.getLazyInitialHooks(getServletContext()).add(this);
    }
  }

  public void lazyInit(ApplicationContext context) {
    setSessionFactory(context.getBean(SessionFactory.class));
  }

  public void doFilter(ServletRequest request, ServletResponse response, FilterChain chain)
      throws ServletException, IOException {
    if (firstEnter(request)) {
      SessionUtils.enableThreadBinding();
      //SessionUtils.openSession(sessionFactory);
      try {
        chain.doFilter(request, response);
      } finally {
        SessionUtils.disableThreadBinding();
        SessionUtils.closeSession(sessionFactory);
      }
    } else {
      chain.doFilter(request, response);
    }
  }

  public void setSessionFactory(SessionFactory sessionFactory) {
    this.sessionFactory = sessionFactory;
  }

}
